@@ -5,3 +5,7 @@ require "numru/gphys/gphys_io"
 require 'numru/gphys/gphys_fft'
 require 'numru/gphys/ep_flux'
 require 'numru/gdir'
+begin
+  require 'numru/gphys_ext'  # extension library, which is yet to be completed
+rescue LoadError
+end
