@@ -575,27 +575,18 @@ module NumRu
 	    info_lost = false
 	    if @@humane_message
 	      if units && /(.*) *since *(.*)/ =~ units  
-		stun = $1
-		tun = Units[stun]
-		since = DateTime.parse($2)
-		if Units[units] =~ Units['days since 1-1-1'] # s,hour,day,..
-		  sec = tun.convert( cval, Units['seconds'] ).round + 1e-1
-			 # ^ take "round" to avoid round error in sec
-			 # (Note that %S below takes floor of seconds).
-		  datetime = since + (sec/86400.0)
+                calendar = self.pos.get_att('calendar')
+                datetime = UNumeric[cval, units].to_datetime(0.1,calendar)
+                if Units[units] =~ Units['days since 1-1-1'] # s,hour,day,..
 		  info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m-%d %H:%M:%S%z')
-		  info_lost_set = true
 		elsif Units[units] =~ Units['months since 1-1-1'] # year,month,pentad,..
-		  monun = Units['months']
-		  if /mon/ =~ stun || /year/ =~ stun
-		    datetime = since >> tun.convert( cval, monun ).round
+		  if /mon/ =~ units.to_s || /year/ =~ units.to_s
 		    info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m')
 		  else  # maybe pentad
-		    datetime = since >> tun.convert( cval, monun )
 		    info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m-%d')
 		  end
-		  info_lost_set = true
-		end
+                end
+		info_lost_set = true
 	      end
 	    end
 	    if !info_lost_set
@@ -684,6 +675,7 @@ module NumRu
 	ax = self.pos.val
 
         units = self.pos.get_att('units')
+        calendar = self.pos.get_att('calendar')
 
 	case coord_cutter
 	when true
@@ -694,8 +686,8 @@ module NumRu
 	  range_ary = [range.first, range.last]
 	  xmin = range_ary.min
 	  xmax = range_ary.max
-          xmin = UNumeric.from_date(xmin,units).val if xmin.class <= Date
-          xmax = UNumeric.from_date(xmax,units).val if xmax.class <= Date
+          xmin = UNumeric.from_date(xmin,units,calendar).val if xmin.class <= Date
+          xmax = UNumeric.from_date(xmax,units,calendar).val if xmax.class <= Date
 	  wh = ( (ax >= xmin) & (ax <= xmax) ).where
 	  if wh.length == 0
 	    raise "Range #{range} does not include any grid point in the axis '#{name}'"
@@ -706,7 +698,7 @@ module NumRu
 	when Numeric, Date, DateTime
 	  # find the nearst point
 	  pt = coord_cutter       
-          pt = UNumeric.from_date(pt,units).val if pt.class <= Date
+          pt = UNumeric.from_date(pt,units,calendar).val if pt.class <= Date
 	  dx = (ax-pt).abs
 	  minloc = _minloc_(dx)
 	  if conserve_rank
@@ -718,7 +710,7 @@ module NumRu
 	  # find the nearst points
 	  ary = coord_cutter
 	  slicer = ary.collect{ |pt|
-            pt = UNumeric.from_date(pt,units).val if pt.class <= Date
+            pt = UNumeric.from_date(pt,units,calendar).val if pt.class <= Date
 	    dx = (ax-pt).abs
 	    minloc = _minloc_(dx)
 	  }
