@@ -9,11 +9,13 @@
 
   * NumRu::Netcdf.copy_global_att  : Copy global attributes from a Gphys 
                                      variable. It operates only when a gphys 
-                                     variable consistof a NetCDF file.
+                                     variable consist of a NetCDF file.
 
 ==HISTORY
 
   2005/08/23  S Takehiro (created)
+  2008/02/14  S Takehiro (copy_global_att can deal with GPhys 
+                          which consists of multiple files)
 
 =end
 
@@ -33,7 +35,14 @@ end
 #------------- Copy global attributes (only for NetCDF file) --------
 class NumRu::NetCDF
   def copy_global_att(gp)
-    ncfile = gp.data.file 
+    ncfiles = gp.data.file 
+    # for gphys consisting of multiple files 
+    if /NArray/  =~ ncfiles.class.to_s  
+      ncfile = ncfiles[0]
+    else
+      ncfile = ncfiles
+    end
+    # Check whether gphys is a NetCDF file or not
     if /NetCDF/  =~ ncfile.class.to_s
       ncfile.each_att{|att|
         att.copy(self)
