@@ -456,6 +456,7 @@ These methods returns a NArray (not a GPhys).
 
 require "numru/gphys/grid"
 require "numru/misc/md_iterators"
+require "numru/gphys/narray_ext"
 
 module NumRu
    class GPhys
@@ -591,8 +592,17 @@ module NumRu
       private :__process_hash_slicer
 
       def cut( *args )
-	newgrid, slicer = @grid.cut( *args )
-	GPhys.new( newgrid, @data[ *slicer ] )
+        if has_assoccoord? && args.length==1 && ((spec=args[0]).is_a?(Hash)) &&
+            ( acnms = (spec.keys & assoccoordnames ) ).length > 0
+          acspec = Hash.new
+          acnms.each{|nm| acspec[nm] = spec.delete(nm)}
+          grid, sl = @grid.cut_assoccoord(acspec)
+          gphys = GPhys.new( grid, self.data[*sl] )
+        else
+          gphys = self
+        end
+	newgrid, slicer = gphys.grid.cut( *args )
+	GPhys.new( newgrid, gphys.data[ *slicer ] )
       end
 
       def cut_rank_conserving( *args )
@@ -623,7 +633,10 @@ module NumRu
 	@grid.axis(i)
       end
       def coord(i)
-	@grid.axis(i).pos
+	@grid.coord(i)
+      end
+      def assoc_coord_gphys(name)
+	@grid.assoc_coord_gphys(name)
       end
       alias coordinate coord
       def lost_axes
@@ -632,6 +645,30 @@ module NumRu
       def dim_index( dimname )
 	 @grid.dim_index( dimname )
       end
+      def coordnames
+        @grid.coordnames
+      end
+      def has_axis?(name)
+        @grid.has_axis?(name)
+      end
+      def has_assoccoord?(*arg)
+        @grid.has_assoccoord?(*arg)
+      end
+      def has_coord?(name)
+        @grid.has_coord?(name)
+      end
+      def assoccoordnames
+        @grid.assoccoordnames
+      end
+      def set_assoc_coords(assoc_crds)
+        @grid.set_assoc_coords(assoc_crds)
+      end
+      def assoc_coords=(assoc_coords)
+        @grid.assoc_coords=assoc_coords
+      end
+      def assoc_coords
+        @grid.assoc_coords
+      end
 
       ## For graphics -->
       def first3D
@@ -930,6 +967,15 @@ module NumRu
 
       end
 
+      def marshal_dump
+        [@data.copy, @grid.copy]
+      end
+
+      def marshal_load(ary)
+        @data = ary[0]
+        @grid = ary[1]
+      end
+
       ############## < private methods > ##############
 
       private
@@ -1118,4 +1164,12 @@ if $0 == __FILE__
      [ sub.mean(0) ]
    end
    p( (gpz3mn - gpz3.mean(0)).abs.max )
+
+   # test marshaling
+   p gpz
+   mar = Marshal.dump(gpz)
+   p mar.class, mar.length
+   g = Marshal.load(mar)
+   p g
+
 end
