@@ -286,6 +286,8 @@ It is intended to behave as a correspondent of a NetCDFVar object.
 require "date"
 require "narray_miss"
 require "numru/gphys/attribute"
+require "numru/gphys/unumeric"
+include Math
 
 module NumRu
  class GrADS_Gridded
@@ -337,6 +339,7 @@ module NumRu
       "little_endian"=>nil,
       "cray_32bit_ieee"=>nil,
       "365_day_calendar"=>nil,
+      "pdef"=>nil,
     }
 
     case(@mode)
@@ -438,7 +441,11 @@ module NumRu
     when "undef"
       att = @undef
     else
-      raise "Invalid/unsupported option: "+key
+      if (@options.has_key?(key))
+        att = @options[key]
+      else
+        raise "Invalid/unsupported option: "+key
+      end
     end
     att
   end
@@ -534,18 +541,13 @@ EOS
       ary_time = get_dim(@dimensions[3])
       start_date = @dimensions[3][:startdatetime]
 
-      case dimensions[3][:increment_units]
-      when 'years'
-        target_date = start_date >> (ary_time[t]*12)
-      when 'months'
-        target_date = start_date >> ary_time[t]
-      when 'days'
-        target_date = start_date + ary_time[t]
-      when 'hours'
-        target_date = start_date + Rational((ary_time[t]*60).to_i,24*60)
-      when 'minutes'
-        target_date = start_date + Rational(ary_time[t].to_i,24*60)
+      if @options["365_day_calendar"] == true
+        calendar = "365_day" 
+      else
+        calendar = nil
       end
+      units = Units["#{dimensions[3][:increment_units]} since #{start_date.strftime('%Y-%m-%d %H:%M:%S')}"]
+      target_date = (UNumeric[ary_time[t],units.to_s]).to_datetime(0.1,calendar)
 
       # substitute DSET by the target file name
       # and determine first record in the target file (="init_xxxx")
@@ -592,25 +594,18 @@ EOS
         @dset_r.gsub!( /%n2/, sprintf("%02d",init_min) )
       end
 
+      # find t in the first record in the target file (="init_t")
+      # and determine the record number in the target file
+
       init_date = DateTime.new(init_year,init_month,init_day,init_hour,init_min)
-      init_t = 0
 
-      ary_time[0..t].to_a.each_with_index{ |time, idx| 
-
-        case dimensions[3][:increment_units]
-        when 'years'
-          before_target_file = ((start_date >> (time*12)) < init_date)
-        when 'months'
-          before_target_file = ((start_date >> time) < init_date)
-        when 'days'
-          before_target_file = ((start_date + time)  < init_date)
-        when 'hours'
-          before_target_file = ((start_date + Rational((time*60).to_i,24*60)) < init_date)
-        when 'minutes'
-          before_target_file = ((start_date + Rational(time.to_i,24*60)) < init_date)
-        end
-        init_t = idx + 1 if before_target_file
-      }
+      init_t = t
+      while init_t > 0
+        time = ary_time[init_t-1]
+        date = (UNumeric[time,units.to_s]).to_datetime(0.1,calendar)
+        break if date < init_date
+        init_t = init_t - 1
+      end
 
       t_in_target_file = t - init_t
 
@@ -775,6 +770,13 @@ EOS
         else
           raise "Invalid line: "+line
         end
+      when /^\s*PDEF/i
+        /^\s*PDEF\s+(\d+)\s+(\d+)\s+(\S+)+(.*)$/i =~ line
+        if ( (isize=$1) && (jsize=$2) && (projection=$3) && (pdefspec=$4))
+          @options["pdef"] = true
+        else
+          raise "Invalid line: "+line
+        end
       when /^\s*[XYZT]DEF/i
 #        /^\s*([XYZT])DEF\s+(\d+)\s+(\S+)\s+(.*)$/i =~ line
         /^\s*([XYZT])DEF\s+(\d+)\s+(\S+)+(.*)$/i =~ line
@@ -891,6 +893,7 @@ EOS
             end
           end
         end
+
         @map=Hash.new
         cum_lev_bytes = 0
         @variables.each{|i|
@@ -904,6 +907,91 @@ EOS
           cum_lev_bytes += i[:nlev]*i[:byte]
         }
       end
+    end
+
+    # overwrite XDEF and YDEF by PDEF if PDEF exists
+    if @options["pdef"] == true
+
+      reference = true
+      case projection
+      when /lcc/i
+        latref,lonref,iref,jref,struelat,ntruelat,slon,dx,dy = pdefspec.split
+
+        imax, jmax = isize.to_i, jsize.to_i
+        dx, dy = dx.to_f, dy.to_f
+        bi, bj = (iref.to_i)-1, (jref.to_i)-1
+        by, bx = latref.to_f, lonref.to_f
+        sy,sx = struelat.to_f, slon.to_f
+        sy2, = ntruelat.to_f
+        unless bx.abs < 180
+          raise "parameter for basepoint is invalid"
+        end
+        unless dx != 0 && dy != 0
+          raise "parameter for distance is invalid"
+        end
+        unless sx.abs < 180
+          raise "parameter for standard is invalid"
+        end
+        unless 0 < sy.abs && sy.abs < sy2.abs && sy2.abs < 90 && sy*sy2>0
+          raise "parameter for standard is invalid"
+        end
+        bx = bx*PI/180
+        by = by*PI/180
+        sx = sx*PI/180
+        sy = sy*PI/180
+        sy2 = sy2*PI/180
+        xi = NArray.sfloat(imax).indgen(1) - bi
+        yj = NArray.sfloat(jmax).indgen(1) - bj
+        x = xi*dx
+        #y = -yj*dy 
+        y = yj*dy
+        tansy = tan(PI/4+sy/2)
+        if sy == sy2
+          n = sin(sy)
+        else
+          tansy2 = tan(PI/4+sy2/2)
+          n = -log(cos(sy)/cos(sy2))/log(tansy/tansy2)
+        end
+        #f = R*cos(sy)*tansy**n/n
+        f = (2.0e7/PI)*cos(sy)*tansy**n/n
+        #r0 = f/tansy**n
+        tansy0 = tan(PI/4+by/2)
+        r0 = f/tansy0**n
+        if reference
+          lon = NMath::asin(sin(n*(bx-sx)) + x/r0)/n + sx
+          r = r0 - y/cos(n*(bx-sx))
+          r = r.abs
+        else
+          rsin = r0*sin(n*(bx-sx)) + x
+          rcos = r0*cos(n*(bx-sx)) - y
+          rcos.reshape!(1,jmax)
+          lon = NMath::atan(rsin/rcos)/n + sx
+          r = NMath::sqrt(rsin**2+rcos**2)
+        end
+        r = -r if n < 0
+        lat = NMath::atan((f/r)**(1.0/n))*2 - PI/2
+        lon = lon*180/PI
+        lat = lat*180/PI
+      else
+        warn "sorry, projection of '#{projection}' is not suported"
+        lon = NArray.sfloat(isize.to_i).indgen(1)
+        lat = NArray.sfloat(jsize.to_i).indgen(1)
+      end
+      p lon, lat
+
+      dim = {:len=>isize.to_i, :flag=>"levels"}
+      dim[:name] = 'x'
+      dim[:description] = 'longitude'
+      dim[:units] = 'degrees_east'
+      dim[:levels] = lon.to_a
+      @dimensions[0] = dim
+
+      dim = {:len=>jsize.to_i, :flag=>"levels"}
+      dim[:name] = 'y'
+      dim[:description] = 'latitude'
+      dim[:units] = 'degrees_north'
+      dim[:levels] = lat.to_a
+      @dimensions[1] = dim
 
     end
 
@@ -1145,6 +1233,9 @@ EOS
       @attr[:name] = @dimensions[0][:name]
       @attr[:long_name] = @dimensions[0][:description]
       @attr[:units] = @dimensions[0][:units]
+      if @attr[:long_name] == "time" && @ctl.get_att("365_day_calendar") == true
+        @attr[:calendar] = "365_day"
+      end
 
       @shape = [@dimensions[0][:len]]
     else
