@@ -201,6 +201,10 @@ module NumRu
           obj.pos = file.pos
 
           is = GribIS.new(obj,file.read(8))
+          unless is.version == 1
+            warn "This Grib version (#{is.version}) has not supported yet"
+            return nil
+          end
           unless is.grib?
             raise "This file is not Grib file (INITIAL SECTION)"
           end
@@ -363,7 +367,7 @@ module NumRu
         @is[4..6] = uint2str(len,3)
         return len
       end
-      def varsion
+      def version
         return str2uint1( @is[7,1] )
       end
       def set_version(ver)
@@ -392,6 +396,8 @@ module NumRu
           else
             if cid==7 # NCEP
               params = PARAMS_NCEP_2
+            elsif cid == 34 # JMA
+              params = PARAMS_JMA
             end
           end
         else
@@ -408,13 +414,13 @@ module NumRu
             if para = NAMES_UNITS[para_id]
               return para
             else
-              $stderr.printf "parameter ID #{nid} in version #{version} has not defined yet\n"
+              warn "parameter ID #{nid} in version #{version} has not defined yet"
             end
           else
-            $stderr.printf "parameter ID #{nid} in version #{version} has not defined yet\n"
+            warn "parameter ID #{nid} in version #{version} has not defined yet"
           end
         else
-          $stderr.printf "parameter table version #{version} has not defined yet\n"
+          warn "parameter table version #{version}, center id #{cid}, version #{version}, parameter id #{nid} has not defined yet"
         end
         return ["unknown (#{nid})", nid.to_s, "-", nil]
       end
@@ -524,7 +530,7 @@ module NumRu
         if zt
           return zt[0]
         else
-          $stderr.printf "z type (#{zid}) is not defined yet\n"
+          warn "z type (#{zid}) is not defined yet"
           return zid.to_s
         end
       end
@@ -538,7 +544,7 @@ module NumRu
         if zn = Z_TYPES[zid]
           return zn[1]
         else
-          $stderr.printf "z type (#{zid}) is not defined yet\n"
+          warn "z type (#{zid}) is not defined yet"
           return "unknown_#{zid}"
         end
       end
@@ -702,7 +708,7 @@ module NumRu
             end
           end
           dlon2 = (lo2-lo1).to_f/(nlon-1)
-          dlon2.round==dlon.round || $stderr.print("Warning: dlon is not same: #{dlon2/1000} != #{dlon.to_f/1000}\n")
+          dlon2.round==dlon.round || warn("Warning: dlon is not same: #{dlon2/1000} != #{dlon.to_f/1000}")
           vlon = NArray.sfloat(nlon).indgen*dlon2+lo1
           if flag_scani==1
             vlon = vlon[-1..0]
@@ -740,8 +746,12 @@ module NumRu
             ie = tmp.eq(tmp.min).where[0]
             vlat = glat[is..ie]
             vlat.length==nlat || raise("length of latitude is not same: #{vlat.length} != #{nlat}")
-            (vlat[0]*1000).round==la1.round || $stderr.printf("Warning: first of latitude is not same: #{vlat[0]} != #{la1.to_f/1000}\n")
-            (vlat[-1]*1000).round==la2.round || $stderr.printf("Warning: last of latitude is not same: #{vlat[-1]} != #{la2.to_f/1000}\n")
+            if (vlat[0]*1000-la1).abs > (vlat[1]*1000-la1).abs
+              warn("Warning: first of latitude is not same: #{vlat[0]} != #{la1.to_f/1000}")
+            end
+            if (vlat[-1]*1000-la2).abs > (vlat[-2]*1000-la2).abs
+              warn("Warning: last of latitude is not same: #{vlat[-1]} != #{la2.to_f/1000}")
+            end
             lat["type"] = "Gaussian latitude"
             lat["number_of_latitude_circles"] = NArray[dlat*2]
           end
@@ -1198,7 +1208,19 @@ module NumRu
           (nlon*nlat)==nlen+nmiss || raise("length is not collect")
           if arg.length!=0
             index, shape2 = arg
-            index = index.collect{|el| Fixnum===el ? el..el : el }
+            index = index.collect{|el| Fixnum===el ? el..el : el}
+            il = index[0]
+            i0 = il.first
+            i1 = il.last
+            i0 = i0 < 0 ? nlon+i0 : i0
+            i1 = i1 < 0 ? nlon+i1 : i1
+            index[0] = il.exclude_end? ? i0...i1 : i0..i1
+            il = index[1]
+            i0 = il.first
+            i1 = il.last
+            i0 = i0 < 0 ? nlat+i0 : i0
+            i1 = i1 < 0 ? nlat+i1 : i1
+            index[1] = il.exclude_end? ? i0...i1 : i0..i1
             j = nil
             i = nil
             if map
@@ -1385,6 +1407,7 @@ module NumRu
         @file.eof? && break
         @file.seek(-1, IO::SEEK_CUR)
         sgm = GribSegment.parse(@file)
+        return nil unless sgm
         name = sgm.pds.sname
         if @vars.has_key?(name)
           @vars[name].push(sgm)
@@ -1623,40 +1646,40 @@ module NumRu
         pds1 = var1.pds
         pds2 = var2.pds
         if pds1.center!=pds2.center
-          $stderr.printf("center is not same: #{pds1.center} != #{pds2.center}\n")
+          warn("center is not same: #{pds1.center} != #{pds2.center}")
           return false
         end
         if pds1.gid!=pds2.gid
-          $stderr.printf("gid is not same: #{pds1.gid} != #{pds2.gid}\n")
+          warn("gid is not same: #{pds1.gid} != #{pds2.gid}")
           return false
         end
         if pds1.unit!=pds2.unit
-          $stderr.printf("unit is not same: #{pds1.unit} != #{pds2.unit}\n")
+          warn("unit is not same: #{pds1.unit} != #{pds2.unit}")
           return false
         end
         zt1 = pds1.z_type
         zt2 = pds2.z_type 
         if zt1 != zt2
           if zt1.nil? || zt2.nil?
-            $stderr.printf("z type is not same: #{zt1 ? zt1 : "nil"} != #{zt2 ? zt2 : "nil"}\n")
+            warn("z type is not same: #{zt1 ? zt1 : "nil"} != #{zt2 ? zt2 : "nil"}")
           else
-            $stderr.printf("z type is not same: #{zt1} != #{zt2}\n")
+            warn("z type is not same: #{zt1} != #{zt2}")
             return false
           end
         end
 =begin
         if pds1.time_range!=pds2.time_range
-          $stderr.printf("time range is not same: #{pds1.time_range} != #{pds2.time_range}\n")
+          warn("time range is not same: #{pds1.time_range} != #{pds2.time_range}")
           return true
 #          return false
         end
 =end
         if pds1.sub_center!=pds2.sub_center
-          $stderr.printf("sub center is not same: #{pds1.sub_center} != #{pds2.sub_center}\n")
+          warn("sub center is not same: #{pds1.sub_center} != #{pds2.sub_center}")
           return false
         end
         if var1.gds!=var2.gds
-          $stderr.printf("gds is not same\n")
+          warn("gds is not same")
           return false
         end
         return true
