@@ -455,6 +455,71 @@ PARAM_WATV_NFLX = 4072
 PARAM_DIV_MST_FLX = 4084
 
 
+## JMA
+PARAM_BVF2 = 5132
+PARAM_SRF_UGRDM = 5136
+PARAM_SRF_VGRDM = 5137
+PARAM_SRF_TMPM = 5138
+PARAM_SRF_SPFHM = 5139
+PARAM_TSC = 5144
+PARAM_TSG = 5145
+PARAM_CWORK = 5146
+PARAM_FGLU = 5147
+PARAM_FGLV = 5148
+PARAM_ADVUA = 5151
+PARAM_VWV = 5152
+PARAM_FGSV = 5154
+PARAM_GFLX = 5155
+PARAM_UWV = 5157
+PARAM_FGSU = 5159
+PARAM_CSUSF = 5160
+PARAM_CSDSF = 5161
+PARAM_CSULF = 5162
+PARAM_CSDLF = 5163
+PARAM_ADVVA = 5165
+PARAM_FRQP = 5168
+PARAM_FRQC = 5169
+PARAM_GWDUA = 5173
+PARAM_GWDVA = 5174
+PARAM_HGTM = 5175
+PARAM_UGRDM = 5176
+PARAM_VGRDM = 5177
+PARAM_VVELM = 5178
+PARAM_TMPM = 5179
+PARAM_SPFHM = 5180
+PARAM_VTU = 5200
+PARAM_VTV = 5201
+PARAM_LTRS = 5202
+PARAM_LINT = 5203
+PARAM_DSWRF = 5204
+PARAM_DLWRF = 5205
+PARAM_USWRF = 5211
+PARAM_ULWRF = 5212
+PARAM_TTHM = 5218
+PARAM_WSMX = 5219
+PARAM_PRMX = 5220
+PARAM_ADVHR = 5222
+PARAM_MSC = 5223
+PARAM_MSG = 5224
+PARAM_MFLXB = 5230
+PARAM_MFLUX = 5231
+PARAM_ADVMR = 5236
+PARAM_OZONE = 5237
+PARAM_CNVUh = 5239
+PARAM_CNVVA = 5240
+PARAM_LRGHR = 5241
+PARAM_CNVHR = 5242
+PARAM_CNVMR = 5243
+PARAM_VDFHR = 5246
+PARAM_VDFUA = 5247
+PARAM_VDFVA = 5248
+PARAM_VDFMR = 5249
+PARAM_SWHR = 5250
+PARAM_LWHR = 5251
+PARAM_LRGMR = 5253
+
+
+
 
 NAMES_UNITS = {
   PARAM_UNKNOWN => ["unknown", "dat", nil, nil],
@@ -863,6 +928,69 @@ NAMES_UNITS = {
   PARAM_DIV_MST_FLX => ["Vertical integral of divergence of moisture flux", "VIDMOF", "kg/m2/s", nil],
 
 
+## JMA
+PARAM_BVF2 => ["Square of Brunt Vaisala frequency", "BVF2", "1/s^2", nil],
+PARAM_SRF_UGRDM => ["Surface 10m zonal wind", "UGRDM", "m/s", nil],
+PARAM_SRF_VGRDM => ["Surface 10m meridional wind", "VGRDM", "m/s", nil],
+PARAM_SRF_TMPM => ["Surface 2m air temperature", "TMPM", "K", nil],
+PARAM_SRF_SPFHM => ["Surface specific humidity", "SPFHM", "kg/kg", nil],
+PARAM_TSC => ["Canopy temperature", "TSC", "K", nil],
+PARAM_TSG => ["Ground temperature", "TSG", "K", nil],
+PARAM_CWORK => ["Cloud work function", "CWORK", "J/kg", nil],
+PARAM_FGLU => ["U sfc momentum flux by long gravity wave", "FGLU", "N/m^2", nil],
+PARAM_FGLV => ["V sfc momentum flux by long gravity wave", "FGLV", "N/m^2", nil],
+PARAM_ADVUA => ["Advective zonal acceleration", "ADVUA", "m/s/day", nil],
+PARAM_VWV => ["Column total of meridional water vapor flux", "VWV", "Kg/m/s", nil],
+PARAM_FGSV => ["V sfc momentum flux by short gravity wave", "FGSV", "N/m^2", nil],
+PARAM_GFLX => ["Ground heat flux", "GFLX", "W/m^2", nil],
+PARAM_UWV => ["Column total of zonal water vapor flux", "UWV", "kg/m/s", nil],
+PARAM_FGSU => ["U sfc momentum flux by short gravity wave", "FGSU", "N/m^2", nil],
+PARAM_CSUSF => ["Clear sky upward solar flux at surface", "CSUSF", "W/m^2", nil],
+PARAM_CSDSF => ["Clear sky downward solar flux at surface", "CSDSF", "W/m^2", nil],
+PARAM_CSULF => ["Clear sky upward long wave flux at top of atm", "CSULF", "W/m^2", nil],
+PARAM_CSDLF => ["Clear sky downward long wave flux at surface", "CSDLF", "W/m^2", nil],
+PARAM_ADVVA => ["Advective meridional acceleration", "ADVVA", "m/s/day", nil],
+PARAM_FRQP => ["Frequency of precipitation", "FRQP", "%", nil],
+PARAM_FRQC => ["Frequency of cumulus precipitation", "FRQC", "%", nil],
+PARAM_GWDUA => ["Gravity wave zonal acceleration", "GWDUA", "m/s/day", nil],
+PARAM_GWDVA => ["Gravity wave meridional acceleration", "GWDVA", "m/s/day", nil],
+PARAM_HGTM => ["Mean geopotential height", "HGTM", "m", nil],
+PARAM_UGRDM => ["Mean zonal wind", "UGRDM", "m/s", nil],
+PARAM_VGRDM => ["Mean meridional wind", "VGRDM", "m/s", nil],
+PARAM_VVELM => ["Mean pressure vertical velocity", "VVELM", "Pa/s", nil],
+PARAM_TMPM => ["Mean temperature", "TMPM", "K", nil],
+PARAM_SPFHM => ["Mean specific humidity", "SPFHM", "kg/kg", nil],
+PARAM_VTU => ["Column total of TU", "VTU", "KPam/s", nil],
+PARAM_VTV => ["Column total of TV", "VTV", "KPam/s", nil],
+PARAM_LTRS => ["Evapotranspiration", "LTRS", "W/m^2", nil],
+PARAM_LINT => ["Interception loss", "LINT", "W/m^2", nil],
+PARAM_DSWRF => ["Downward solar radiation flux at surface", "DSWRF", "W/m^2", nil],
+PARAM_DLWRF => ["Downward long wave radiation flux at surface", "DLWRF", "W/m^2", nil],
+PARAM_USWRF => ["Upward solar radiation at surface", "USWRF", "W/m^2", nil],
+PARAM_ULWRF => ["Upward long wave radiation flux at surface", "ULWRF", "W/m^2", nil],
+PARAM_TTHM => ["Column total of diabatic heat", "TTHM", "W/m^2", nil],
+PARAM_WSMX => ["Surface 10m maximum surface wind", "WSMX", "m/s", nil],
+PARAM_PRMX => ["Maximum hourly precipitation", "PRMX", "mm/hour", nil],
+PARAM_ADVHR => ["Advective heating rate", "ADVHR", "K/day", nil],
+PARAM_MSC => ["Canopy water content", "MSC", "mm", nil],
+PARAM_MSG => ["Ground water content", "MSG", "mm", nil],
+PARAM_MFLXB => ["Upward mass flux at cloud base", "MFLXB", "kg/m^2/s", nil],
+PARAM_MFLUX => ["Upward mass flux", "MFLUX", "kg/m^2/s", nil],
+PARAM_ADVMR => ["Advective moistening rate", "ADVMR", "kg/kg/day", nil],
+PARAM_OZONE => ["Ozone mass mixing ratio", "OZONE", "ppm", nil],
+PARAM_CNVUh => ["Convective zonal acceleration", "CNVUh", "m/s/day", nil],
+PARAM_CNVVA => ["Convective meridional acceleration", "CNVVA", "m/s/day", nil],
+PARAM_LRGHR => ["Large scale condensation heating rate", "LRGHR", "K/day", nil],
+PARAM_CNVHR => ["Convective heating rate", "CNVHR", "k/day", nil],
+PARAM_CNVMR => ["Convective moistening rate", "CNVMR", "kg/kg/s", nil],
+PARAM_VDFHR => ["Vertical diffusion heating rate", "VDFHR", "K/day", nil],
+PARAM_VDFUA => ["Vertical diffusion zonal acceleration", "VDFUA", "m/s/day", nil],
+PARAM_VDFVA => ["Vertical diffusion meridional acceleration", "VDFVA", "m/s/day", nil],
+PARAM_VDFMR => ["Vertical diffusion moistening rate", "VDFMR", "kg/kg/day", nil],
+PARAM_SWHR => ["Solar radiative heating rate", "SWHR", "k/day", nil],
+PARAM_LWHR => ["Longwave radiative heating rate", "LWHR", "K/day", nil],
+PARAM_LRGMR => ["Large scale moistening rate", "LRGMR", "kg/kg/day", nil],
+
 nil => nil
 }
 
@@ -1268,3 +1396,68 @@ PARAMS_ECMWF_162 = {   #inconmplte
   72 => PARAM_WATV_NFLX,
   84 => PARAM_DIV_MST_FLX
 }
+
+
+PARAMS_JMA = {
+  132 => PARAM_BVF2,
+  136 => PARAM_SRF_UGRDM,
+  137 => PARAM_SRF_VGRDM,
+  138 => PARAM_SRF_TMPM,
+  139 => PARAM_SRF_SPFHM,
+  144 => PARAM_TSC,
+  145 => PARAM_TSG,
+  146 => PARAM_CWORK,
+  147 => PARAM_FGLU,
+  148 => PARAM_FGLV,
+  151 => PARAM_ADVUA,
+  152 => PARAM_VWV,
+  154 => PARAM_FGSV,
+  155 => PARAM_GFLX,
+  157 => PARAM_UWV,
+  159 => PARAM_FGSU,
+  160 => PARAM_CSUSF,
+  161 => PARAM_CSDSF,
+  162 => PARAM_CSULF,
+  163 => PARAM_CSDLF,
+  165 => PARAM_ADVVA,
+  168 => PARAM_FRQP,
+  169 => PARAM_FRQC,
+  173 => PARAM_GWDUA,
+  174 => PARAM_GWDVA,
+  175 => PARAM_HGTM,
+  176 => PARAM_UGRDM,
+  177 => PARAM_VGRDM,
+  178 => PARAM_VVELM,
+  179 => PARAM_TMPM,
+  180 => PARAM_SPFHM,
+  200 => PARAM_VTU,
+  201 => PARAM_VTV,
+  202 => PARAM_LTRS,
+  203 => PARAM_LINT,
+  204 => PARAM_DSWRF,
+  205 => PARAM_DLWRF,
+  211 => PARAM_USWRF,
+  212 => PARAM_ULWRF,
+  218 => PARAM_TTHM,
+  219 => PARAM_WSMX,
+  220 => PARAM_PRMX,
+  222 => PARAM_ADVHR,
+  223 => PARAM_MSC,
+  224 => PARAM_MSG,
+  230 => PARAM_MFLXB,
+  231 => PARAM_MFLUX,
+  236 => PARAM_ADVMR,
+  237 => PARAM_OZONE,
+  239 => PARAM_CNVUh,
+  240 => PARAM_CNVVA,
+  241 => PARAM_LRGHR,
+  242 => PARAM_CNVHR,
+  243 => PARAM_CNVMR,
+  246 => PARAM_VDFHR,
+  247 => PARAM_VDFUA,
+  248 => PARAM_VDFVA,
+  249 => PARAM_VDFMR,
+  250 => PARAM_SWHR,
+  251 => PARAM_LWHR,
+  253 => PARAM_LRGMR,
+}
