@@ -1,4 +1,5 @@
 require "numru/gphys/axis"
+require "numru/gphys/assoccoords"
 
 =begin
 =class NumRU::Grid
@@ -139,15 +140,20 @@ A class to handle discretized grids of physical quantities.
     Similar to ((<cut>)), but the rank is conserved by not eliminating
     any dimension (whose length could be one).
 
----exclude(dim_or_dimname)
-    Returns a Grid in which an axis is eliminated from self.
-
-    ARGUMENTS
-    * dim_or_dimname (String or Integer) to specify an axis.
-
-    RETURN VALUE
-    * a Grid
+---cut_assoccoord(hasharg)
+    Cut with respect to the asscoated coordinates. Similar to cut,
+    when the argeuemnt is a Hash, but hash keys are the names of the 
+    asscoated coordinates.
 
+#---exclude(dim_or_dimname)
+#    Returns a Grid in which an axis is eliminated from self.
+#
+#    ARGUMENTS
+#    * dim_or_dimname (String or Integer) to specify an axis.
+#
+#    RETURN VALUE
+#    * a Grid
+#
 ---change_axis(dim, axis)
     Replaces an axis. (Returns a new object)
 
@@ -157,36 +163,36 @@ A class to handle discretized grids of physical quantities.
 
     RETURN VALUE
     * a Grid
-
----change_axis!(dim_or_dimname, axis)
-    Replaces an axis. (overwrites self)
-
-    ARGUMENTS
-    * dim_or_dimname (String or Integer) to specify an axis.
-    * axis (Axis)
-
-    RETURN VALUE
-    * self
-
----insert_axis(dim_or_dimname, axis)
-    Inserts an axis. (Returns a new object)
-
-    ARGUMENTS
-    * dim_or_dimname (String or Integer) to specify an axis.
-    * axis (Axis)
-
-    RETURN VALUE
-    * a Grid
-
----insert_axis!(dim_or_dimname, axis)
-    Inserts an axis. (overwrites self)
-
-    ARGUMENTS
-    * dim_or_dimname (String or Integer) to specify an axis.
-    * axis (Axis)
-
-    RETURN VALUE
-    * self
+#
+#---change_axis!(dim_or_dimname, axis)
+#    Replaces an axis. (overwrites self)
+#
+#    ARGUMENTS
+#    * dim_or_dimname (String or Integer) to specify an axis.
+#    * axis (Axis)
+#
+#    RETURN VALUE
+#    * self
+#
+#---insert_axis(dim_or_dimname, axis)
+#    Inserts an axis. (Returns a new object)
+#
+#    ARGUMENTS
+#    * dim_or_dimname (String or Integer) to specify an axis.
+#    * axis (Axis)
+#
+#    RETURN VALUE
+#    * a Grid
+#
+#---insert_axis!(dim_or_dimname, axis)
+#    Inserts an axis. (overwrites self)
+#
+#    ARGUMENTS
+#    * dim_or_dimname (String or Integer) to specify an axis.
+#    * axis (Axis)
+#
+#    RETURN VALUE
+#    * self
 
 ---transpose( *dims )
     Transpose.
@@ -217,10 +223,28 @@ module NumRu
 	 @rank = @axes.length
 	 @axnames = Array.new
 	 __check_and_set_axnames
+         @assoc_coords = nil
+      end
+
+      # * assoc_crds : an Array of GPhys
+      # 
+      def set_assoc_coords(assoc_crds)
+        @assoc_coords = AssocCoords.new(assoc_crds, @axnames)
+        self
+      end
+
+      # * assoc_coords : an AssocCoords
+      # 
+      def assoc_coords=(assoc_coords)
+        @assoc_coords = assoc_coords 
+      end
+
+      def assoc_coords
+        @assoc_coords
       end
 
       def inspect
-	 "<#{rank}D grid #{@axes.collect{|ax| ax.inspect}.join("\n\t")}>"
+        "<#{rank}D grid #{@axes.collect{|ax| ax.inspect}.join("\n\t")}#{@assoc_coords ? "\n"+@assoc_coords.inspect.gsub(/^/,"\t") : ''}>"
       end
 
       def __check_and_set_axnames
@@ -244,6 +268,37 @@ module NumRu
 	@lost_axes.dup
       end
 
+      def coordnames
+        ret = axnames
+        ret.concat(@assoc_coords.coordnames) if @assoc_coords
+        ret
+      end
+
+      def has_axis?(name)
+	@axnames.include?(name)
+      end
+
+      def has_assoccoord?(*arg)
+        if arg.length == 0
+          !@assoc_coords.nil?   # if a grid has assoc coords
+        else
+          name = arg[0]
+          if @assoc_coords
+            @assoc_coords.has_coord?(name)
+          else
+            false
+          end
+        end
+      end
+
+      def has_coord?(name)
+        has_axis?(name) || has_assoccoord?(name)
+      end
+
+      def assoccoordnames
+	@assoc_coords && @assoc_coords.coordnames
+      end
+
 #      def axis(i)
 #	 @axes[i]
 #      end
@@ -254,12 +309,24 @@ module NumRu
 
       alias get_axis axis
 
+      def coord(i)
+        if @assoc_coords && i.is_a?(String) && @assoc_coords.has_coord?(i)
+          @assoc_coords.coord(i)
+        else
+          axis(i).pos
+        end
+      end
+
+      def assoc_coord_gphys(name)
+        @assoc_coords && @assoc_coords.coord_gphys(name)
+      end
+
       def dim_index(dimname)
 	 ax_dim(dimname)[1]
       end
 
       def set_axis(dim_or_dimname,ax)
-	 @axes[ ax_dim(dim_or_dimname)[1] ] = ax
+	 @axes[ dim_index(dim_or_dimname) ] = ax
 	 self
       end
 
@@ -311,6 +378,9 @@ module NumRu
 
 	newgrid = self.class.new( *newaxes )
 	newgrid.set_lost_axes( @lost_axes.dup )
+        if @assoc_coords
+          newgrid.assoc_coords=@assoc_coords.subset_having_axnames(newgrid.axnames)
+        end
 
 	if !deleted_by 
 	  msg = '(deleted) '
@@ -333,6 +403,7 @@ module NumRu
 	 # deep clone onto memory
 	 out = self.class.new( *@axes.collect{|ax| ax.copy} )
 	 out.set_lost_axes( @lost_axes.dup )
+         out.assoc_coords = @assoc_coords.copy if @assoc_coords
 	 out
       end
 
@@ -353,6 +424,10 @@ module NumRu
 	end
 	out = self.class.new( *axes )
 	out.set_lost_axes( (@lost_axes.dup + other.lost_axes).uniq )
+        if (oac = other.assoc_coords) || (sac = self.assoc_coords)
+          sac ? ac=sac.merge(oac) : ac=oac.merge(sac)
+          out.assoc_coords = ac
+        end
 	out
       end
 
@@ -366,7 +441,7 @@ module NumRu
 	    # make a clone
 	    axes = Array.new
 	    (0...rank).each{ |i| axes.push( @axes[i][0..-1] ) }
-	    self.class.new( *axes )
+	    grid = self.class.new( *axes )
 	 else
 	    slicer = __rubber_expansion(slicer)
 	    if slicer.length != rank
@@ -386,6 +461,10 @@ module NumRu
 	    grid.set_lost_axes( lost ) if lost.length != 0
 	    grid
 	 end
+         if @assoc_coords
+           grid.assoc_coords = @assoc_coords[*slicer]
+         end
+         grid
       end
 
       def __rubber_expansion( args )
@@ -404,6 +483,12 @@ module NumRu
       end
       private :__rubber_expansion
 
+      def cut_assoccoord(hasharg)
+        ac, acsl = @assoc_coords.cut(hasharg)
+        sl = @axnames.collect{|nm| acsl[nm] || true}
+        [ self[*sl], sl ]
+      end
+
       def cut(*args)
 	_cut_(false, *args)
       end
@@ -411,11 +496,13 @@ module NumRu
 	_cut_(true, *args)
       end
 
+      # cut along the regular coorinates.
+      # assume that the coordinates are monotonic (without checking).
       def _cut_(conserve_rank, *args)
 
-	# assume that the coordinates are monotonic (without checking)
+        hasharg = ( args.length==1 && args[0].is_a?(Hash) )
 
-	if args.length==1 && args[0].is_a?(Hash)
+	if hasharg
 	  # specification by axis names
 	  spec = args[0]
 	  if (spec.keys - axnames).length > 0
@@ -447,12 +534,12 @@ module NumRu
       end
       private :_cut_
 
-      def exclude(dim_or_dimname)
-	 dim = dim_index(dim_or_dimname)
-	 axes = @axes.dup
-	 axes.delete_at(dim)
-	 self.class.new( *axes )
-      end
+#      def exclude(dim_or_dimname)
+#	 dim = dim_index(dim_or_dimname)
+#	 axes = @axes.dup
+#	 axes.delete_at(dim)
+#	 self.class.new( *axes )
+#      end
 
       def change_axis(dim, axis)
 	 axes = @axes.dup
@@ -463,43 +550,47 @@ module NumRu
 	    lost.push(axis) if axis.is_a?(String)
 	    axes.delete_at(dim)
 	 end
-	 self.class.new( *axes ).add_lost_axes( lost )
+	 grid = self.class.new( *axes ).add_lost_axes( lost )
+         if @assoc_coords
+           grid.assoc_coords=@assoc_coords.subset_having_axnames(grid.axnames)
+         end
+         grid
       end
 
-      def change_axis!(dim_or_dimname, axis)
-	 if axis.is_a?(Axis)
-	    @axes[ ax_dim(dim_or_dimname)[1] ] = axis
-	 else
-	    @lost_axes.push(axis) if axis.is_a?(String)
-	    @axes.delete_at( ax_dim(dim_or_dimname)[1] )
-	 end
-	 @rank = @axes.length
-	 __check_and_set_axnames
-	 self
-      end
+#      def change_axis!(dim_or_dimname, axis)
+#	 if axis.is_a?(Axis)
+#	    @axes[ dim_index(dim_or_dimname) ] = axis
+#	 else
+#	    @lost_axes.push(axis) if axis.is_a?(String)
+#	    @axes.delete_at( dim_index(dim_or_dimname) )
+#	 end
+#	 @rank = @axes.length
+#	 __check_and_set_axnames
+#	 self
+#      end
 
-      def insert_axis(dim_or_dimname, axis)
-	 dim = ax_dim(dim_or_dimname)[1]
-	 axes = @axes.dup
-	 if axis.is_a?(Axis)
-	    axes[dim+1,0] = axis    # axes.insert(dim, axis) if ruby 1.7
-	 else
-	    # do nothing
-	 end
-	 self.class.new( *axes )
-      end
+#      def insert_axis(dim_or_dimname, axis)
+#	 dim = dim_index(dim_or_dimname)
+#	 axes = @axes.dup
+#	 if axis.is_a?(Axis)
+#	    axes[dim+1,0] = axis    # axes.insert(dim, axis) if ruby 1.7
+#	 else
+#	    # do nothing
+#	 end
+#	 self.class.new( *axes )
+#      end
 
-      def insert_axis!(dim_or_dimname, axis)
-	 dim = ax_dim(dim_or_dimname)[1]
-	 if axis == nil
-	    # do nothing
-	 else
-	    @axes[dim+1,0] = axis    # @axes.insert(dim, axis) if ruby 1.7
-	    @rank = @axes.length
-	    __check_and_set_axnames
-	 end
-	 self
-      end
+#      def insert_axis!(dim_or_dimname, axis)
+#	 dim = dim_index(dim_or_dimname)
+#	 if axis == nil
+#	    # do nothing
+#	 else
+#	    @axes[dim+1,0] = axis    # @axes.insert(dim, axis) if ruby 1.7
+#	    @rank = @axes.length
+#	    __check_and_set_axnames
+#	 end
+#	 self
+#      end
 
       def transpose( *dims )
 	if dims.sort != NArray.int(rank).indgen!.to_a
@@ -512,6 +603,7 @@ module NumRu
 	end
 	grid = self.class.new(*axes)
 	grid.set_lost_axes( lost_axes )
+        grid.assoc_coords = @assoc_coords.dup if @assoc_coords
 	grid
       end
 
@@ -597,6 +689,5 @@ if $0 == __FILE__
    p grid.delete_axes([0,1], 'mean').lost_axes
 
    p grid, grid.transpose(1,0)
-
 end
 
