@@ -167,7 +167,8 @@ module NumRu
 	when /^Wind Profiler/
 	  NetCDF_Convention_Wind_Profiler
 	else
-	  NetCDF_Convention_Users_Guide
+	  #NetCDF_Convention_Users_Guide
+          NetCDF_Convention_CF
 	end
       end
     end
@@ -299,8 +300,47 @@ module NumRu
       nil   # no rule for that
     end
 
+    def assoc_coord_names(data)
+      nil   # no rule for that
+    end
+
   end
 
+  module NetCDF_Convention_CF
+    # http://cf-pcmdi.llnl.gov/
+
+    # < inherit the Users guide convention >
+
+    module Attribute_Mixin
+      include NetCDF_Convention_Users_Guide::Attribute_Mixin
+    end    # module Attribute_Mixin
+    module VArray_Mixin
+      include NetCDF_Convention_Users_Guide::VArray_Mixin
+    end    # module VArray_Mixin
+    module_function
+    extend NetCDF_Convention_Users_Guide
+    public_class_method :aux_var_names, :coord_var_names, :cell_bounds?, :cell_center?
+
+    ########### original part starts here #################
+
+    def assoc_coord_names(data)
+      if s = data.get_att("coordinates")
+        nms = s.split(/ +/)
+        case data.file
+        when NArray
+          nms.delete_if{|nm| !data.file[0].var(nm)}
+        else
+          nms.delete_if{|nm| !data.file.var(nm)}
+        end
+        nms
+      else
+        nil
+      end
+    end
+
+  end
+
+
   module NetCDF_Convention_Wind_Profiler
     # Wind Profiler Convention
     # http://www.kurasc.kyoto-u.ac.jp/radar-group/wind_profiler_conventions/
@@ -316,7 +356,7 @@ module NumRu
     ########################
     module_function
     extend NetCDF_Convention_Users_Guide
-    public_class_method :cell_bounds?, :cell_center?
+    public_class_method :cell_bounds?, :cell_center?, :assoc_coord_names
     
     def to_s
       "Wind Profiler (http://www.kurasc.kyoto-u.ac.jp/radar-group/wind_profiler_conventions/)"
