@@ -189,7 +189,7 @@ module NumRu
 
     # * date (Date or DateTime)
     # * units (Units or String) : units of the UNumeric to be created
-    def self::from_date(date, units)
+    def self::from_date(date, units, calendar=nil)
       sunits = units.to_s
       /(.*) *since *(.*)/ =~ sunits
       if (!$1 or !$2)
@@ -202,7 +202,28 @@ module NumRu
         year,mon = date.year,date.mon
         time = Units['months'].convert((year*12+mon)-(year0*12+mon0), tun)
       elsif( tun =~ Units['days since 0001-01-01'] )
-        time = Units['days'].convert( date-since, tun )
+        case calendar
+        when nil
+          time = Units['days'].convert( date-since, tun )
+        when "noleap", "365_day"
+          since_yday = since - DateTime.new(since.year,1,1) # day number of year (0..364)
+          since_yday = since_yday - 1 if( since.leap? && since.mon > 2 )
+          date_yday = date - DateTime.new(date.year,1,1)
+          if( date.leap? )
+            if date_yday >= 60.0 # after Mar1
+              date_yday = date_yday - 1
+            elsif date_yday >= 59.0 # Feb29
+              raise "Feb.29 is specified, but calendar is #{calendar}."
+            end
+          end
+          days = (date.year - since.year)*365 + (date_yday - since_yday)
+          time = Units['days'].convert( days, tun )
+        when "360_day" # does not work perfectly
+          days = (date.year - since.year)*360 + (date.mon - since.mon)*30 + (date.day - since.day)
+          time = Units['days'].convert( days, tun )
+        else
+          raise("Unrecognized calendar: #{calendar}")
+        end
       else
         raise("Unrecognized time units #{tun.to_s} -- may be a BUG?")
       end
@@ -250,7 +271,7 @@ module NumRu
 
     # * eps_sec : Magic epsilon to prevent the round-off of DateTime [seconds].
     #             Recommended value is 0.1.
-    def to_datetime(eps_sec=0.0)
+    def to_datetime(eps_sec=0.0,calendar=nil)
       time = self.val
       sunits = self.units.to_s
       /(.*) *since *(.*)/ =~ sunits
@@ -262,7 +283,26 @@ module NumRu
       if( tun =~ Units['months since 0001-01-01'] )
         datetime = since >> tun.convert( time, Units['months'] )
       elsif( tun =~ Units['days since 0001-01-01'] )
-        datetime = since + tun.convert( time, Units['days'] )
+        case calendar
+        when nil
+          datetime = since + tun.convert( time, Units['days'] )
+        when "noleap", "365_day"
+          since_yday = since - DateTime.new(since.year,1,1) # day number of year (0..364)
+          since_yday = since_yday - 1 if( since.leap? && since.mon > 2 )
+          days = since_yday + tun.convert( time, Units['days'] )
+          year = since.year + (days/365).to_i
+          date_yday = days%365 # day number of year (0..364)
+          datetime = DateTime.new(year,1,1) + date_yday
+          datetime = datetime + 1 if( datetime.leap? && date_yday >= 59 )
+        when "360_day" # does not work perfectly
+          since_day = since - DateTime.new(since.year,since.mon,1)
+          days = (since.mon-1)*30 + since_day + tun.convert( time, Units['days'] )
+          year = since.year + (days/360).to_i
+          mon = ((days%360)/30).to_i + 1
+          datetime = DateTime.new(year,mon,1) + days%30
+        else
+          raise("Unrecognized calendar: #{calendar}")
+        end
       else
         raise("Unrecognized time units #{tun.to_s} -- may be a BUG?")
       end
