@@ -325,7 +325,7 @@ These methods returns a NArray (not a VArray).
 	 when nil
 	    @attr = NumRu::Attribute.new
 	 else
-	   raise TypeErroor, "#{attr.class} is unsupported for the 2nd arg"
+	   raise TypeError, "#{attr.class} is unsupported for the 2nd arg"
 	 end
       end
 
@@ -409,11 +409,7 @@ These methods returns a NArray (not a VArray).
       end
 
       def ntype
-	 if !@mapping
-	    __ntype(@ary)
-	 else
-	    __ntype(@varray.ary)
-	 end
+        __ntype(typecode)
       end
 
       def name=(nm)
@@ -446,7 +442,7 @@ These methods returns a NArray (not a VArray).
 	if @mapping
 	  @varray.file
 	else
-	  raise nil
+	  return nil
 	end
       end
 
@@ -998,8 +994,8 @@ These methods returns a NArray (not a VArray).
 	 end
 	 narray
       end
-      def __ntype(na)
-	 case na.typecode
+      def __ntype(typecode)
+	 case typecode
 	 when NArray::BYTE
 	    "byte"
 	 when NArray::SINT
@@ -1076,4 +1072,7 @@ if $0 == __FILE__
    vx.set_att('modulo',[360.0])
    vx.set_att('positive','down')
    p vx.axis_draw_positive, vx.axis_cyclic?, vx.axis_modulo
+
+   p '*typecode*'
+   p vx.typecode, vx[0..1].typecode
 end
