@@ -225,7 +225,7 @@ module NumRu
     end
 
     def ntype
-      __ntype(@first_vary)
+      __ntype(typecode)
     end
 
     def typecode
@@ -412,4 +412,5 @@ if __FILE__ == $0
   vac.set_att('long_name', 'test data')
   p vac.get_att('long_name')
   p va01.get_att('long_name')
+  p vac.typecode,vac.ntype
 end
